(*                                                                      *)
(* MLJVM - a Java Virtual Machine implemented in Standard ML            *)
(*                                                                      *)
(* http://homepages.inf.ed.ac.uk/stg/research/projects/MLJVM/           *)
(*                                                                      *)
(* Copyright (C) 2003 Stephen Gilmore and Laura Korte, The              *)
(* University of Edinburgh.                                             *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*                                                                      *)

structure Bytecode2 =
struct

    datatype atype =
        Abool  | Achar 
      | Afloat | Adouble
      | Abyte  | Ashort
      | Aint   | Along

    datatype offset = OFF of int
    
    type cp_index   = Constpool.index
    type var_index  = Localvar.index
    type byte       = int
    type short      = int

    (*The type of JVM instructions*)
    datatype jvm_instr =
        Jempty (*placeholder*)
      | Jaaload
      | Jaastore
      | Jaconst_null
      | Jaload of var_index
      | Jaload_0 | Jaload_1 | Jaload_2 | Jaload_3   
      | Janewarray of cp_index
      | Jareturn   
      | Jarraylength
      | Jastore of var_index
      | Jastore_0 | Jastore_1 | Jastore_2 | Jastore_3   
      | Jathrow
      | Jbaload
      | Jbastore
      | Jbipush of byte
      | Jcaload
      | Jcastore
      | Jcheckcast of cp_index
      | Jd2f
      | Jd2i
      | Jd2l
      | Jdadd
      | Jdaload
      | Jdastore
      | Jdcmpg | Jdcmpl
      | Jdconst_0 | Jdconst_1   
      | Jddiv
      | Jdload of var_index
      | Jdload_0 | Jdload_1 | Jdload_2 | Jdload_3   
      | Jdmul
      | Jdneg
      | Jdrem
      | Jdreturn   
      | Jdstore of var_index
      | Jdstore_0 | Jdstore_1 | Jdstore_2 | Jdstore_3   
      | Jdsub
      | Jdup
      | Jdup_x1
      | Jdup_x2
      | Jdup2
      | Jdup2_x1
      | Jdup2_x2
      | Jf2d
      | Jf2i
      | Jf2l
      | Jfadd
      | Jfaload
      | Jfastore
      | Jfcmpg | Jfcmpl
      | Jfconst_0 | Jfconst_1 | Jfconst_2   
      | Jfdiv
      | Jfload of var_index
      | Jfload_0 | Jfload_1 | Jfload_2 | Jfload_3   
      | Jfmul
      | Jfneg
      | Jfrem
      | Jfreturn   
      | Jfstore of var_index
      | Jfstore_0 | Jfstore_1 | Jfstore_2 | Jfstore_3    
      | Jfsub
      | Jgetfield of cp_index
      | Jgetstatic of cp_index
      | Jgoto of offset
      | Jgoto_w of offset
      | Ji2b
      | Ji2c
      | Ji2d
      | Ji2f
      | Ji2l
      | Ji2s
      | Jiadd
      | Jiaload
      | Jiand
      | Jiastore
      | Jiconst_m1 | Jiconst_0 | Jiconst_1 | Jiconst_2 | Jiconst_3 | Jiconst_4 | Jiconst_5   
      | Jidiv
      | Jif_acmpeq of offset | Jif_acmpne of offset
      | Jif_icmpeq of offset | Jif_icmpne of offset | Jif_icmplt of offset
      | Jif_icmpge of offset | Jif_icmpgt of offset | Jif_icmple of offset
      | Jifeq of offset | Jifne of offset | Jiflt of offset
      | Jifge of offset | Jifgt of offset | Jifle of offset
      | Jifnonnull of offset | Jifnull of offset
      | Jiinc of { var   : var_index
                 , const : byte }
      | Jiload of var_index
      | Jiload_0 | Jiload_1 | Jiload_2 | Jiload_3   
      | Jimul
      | Jineg
      | Jinstanceof of cp_index
      | Jinvokeinterface of cp_index
      | Jinvokespecial of cp_index
      | Jinvokestatic of cp_index
      | Jinvokevirtual of cp_index
      | Jior
      | Jirem
      | Jireturn   
      | Jishl
      | Jishr
      | Jistore of var_index
      | Jistore_0 | Jistore_1 | Jistore_2 | Jistore_3   
      | Jisub
      | Jiushr
      | Jixor
      | Jjsr of offset
      | Jjsr_w of offset
      | Jl2d
      | Jl2f
      | Jl2i
      | Jladd
      | Jlaload
      | Jland
      | Jlastore
      | Jlcmp
      | Jlconst_0 | Jlconst_1   
      | Jldc of    cp_index
      | Jldc_w of  cp_index   
      | Jldc2_w of cp_index
      | Jldiv
      | Jlload of var_index
      | Jlload_0 | Jlload_1 | Jlload_2 | Jlload_3   
      | Jlmul
      | Jlneg
      | Jlookupswitch of { default : offset
                         , cases   : (Int32.int * offset) list }
      | Jlor
      | Jlrem
      | Jlreturn   
      | Jlshl
      | Jlshr
      | Jlstore of var_index
      | Jlstore_0 | Jlstore_1 | Jlstore_2 | Jlstore_3    
      | Jlsub
      | Jlushr
      | Jlxor
      | Jmonitorenter
      | Jmonitorexit
      | Jmultianewarray of { elem : cp_index
                           , dim  : int }
      | Jnew of cp_index
      | Jnewarray of atype   
      | Jnop
      | Jpop
      | Jpop2
      | Jputfield of cp_index
      | Jputstatic of cp_index
      | Jret of var_index
      | Jreturn
      | Jsaload
      | Jsastore
      | Jsipush of short
      | Jswap
      | Jtableswitch of { default : offset
                        , low     : int
			, high    : int
			, offsets : offset Vector.vector }
      | Jwide of jvm_instr
      | Nprintln
      | Nprint
      | Nint2string
      | NparseInt

end;
