(*                                                                      *)
(* MLJVM - a Java Virtual Machine implemented in Standard ML            *)
(*                                                                      *)
(* http://homepages.inf.ed.ac.uk/stg/research/projects/MLJVM/           *)
(*                                                                      *)
(* Copyright (C) 2003 Stephen Gilmore and Laura Korte, The              *)
(* University of Edinburgh.                                             *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*                                                                      *)


structure Environment =
struct

exception Error of string;

type word = int

datatype address = NATIVE of word | OFFSET of word  
datatype object_pointer = OP of word

fun addrValue (NATIVE word) = word
  | addrValue (OFFSET word) = word
fun add (NATIVE word,n) = NATIVE (word + n)
  | add (OFFSET word,n) = OFFSET (word + n)

datatype stack_item = Sempty (*placeholder*)
                    | Sint of Int32.int
                    | Sfloat of Real32.real
		    | Sreference of Heap.hpointer
		    | SreturnAddress of address
		    | Slong of Int64.int     (*occupies 2 stack entries*)
		    | Sdouble of Real64.real (*occupies 2 stack entries*)

datatype frame = FRAME of { stack          : stack_item list
                          , local_vars     : (stack_item option) Array.array
	   	          , current_class  : RuntimeClass.index
	   	          , current_pc     : address
	   	          , current_code   : Bytecode2.jvm_instr Array.array
			  , current_method : RuntimeClass.mindex }

type stack = frame list

datatype environment = ENV of { method_area    : RuntimeClass.method_area      (*shared*)
                              , method_counter : int                           (*shared*)
                              , heap           : Heap.heap                     (*shared*)
			      , stack          : stack                     (*no threads*)
			      , pc_register    : address                   (*no threads*) }

end;
