(*                                                                      *)
(* MLJVM - a Java Virtual Machine implemented in Standard ML            *)
(*                                                                      *)
(* http://homepages.inf.ed.ac.uk/stg/research/projects/MLJVM/           *)
(*                                                                      *)
(* Copyright (C) 2003 Stephen Gilmore and Laura Korte, The              *)
(* University of Edinburgh.                                             *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*                                                                      *)

structure Modifiers =
struct

   exception UnknownFlagError of int;

   datatype access_flag =
       ACCpublic	(* field/method/class/interface *)
     | ACCprivate	(* field/method *)
     | ACCprotected	(* field/method *)
     | ACCstatic	(* field/method *)
     | ACCfinal		(* field/method/class *)
     | ACCvolatile	(* field *)
     | ACCtransient	(* field *)
     | ACCsynchronized	(* method *)
     | ACCnative	(* method *)
     | ACCabstract	(* method/class/interface *)
     | ACCsuper		(* class/interface *)
     | ACCinterface	(* interface *)
     | ACCstrictfp

   val public        = 0x0001;
   val private       = 0x0002;
   val protected     = 0x0004;
   val static        = 0x0008;
   val final         = 0x0010;
   val super         = 0x0020;
   val synchronized  = 0x0020;
   val volatile      = 0x0040;
   val transient     = 0x0080;
   val native        = 0x0100;
   val interface     = 0x0200;
   val abstract      = 0x0400;
   val strictfp      = 0x0800;

   val priority_list = [ strictfp
                       , abstract
		       , interface
		       , native
		       , transient
		       , volatile
		       , synchronized
		       , super
		       , final
		       , static
		       , protected
		       , private
		       , public ]
		       
   fun class_mod m =
       case m of 0x0001 => "public"
	       | 0x0010 => "final"
	       | 0x0020 => "super"
	       | 0x0200 => "interface"
	       | 0x0400 => "abstract"
	       | 0x0800 => "strictfp"
	       | n      => raise UnknownFlagError n

   fun method_mod m =
       case m of 0x0001 => "public"
	       | 0x0002 => "private"
	       | 0x0004 => "protected"
	       | 0x0008 => "static"
	       | 0x0010 => "final"
	       | 0x0020 => "synchronized"
	       | 0x0040 => "volatile"
	       | 0x0080 => "transient"
	       | 0x0100 => "native"
	       | 0x0200 => "interface"
	       | 0x0400 => "abstract"
	       | 0x0800 => "strictfp"
	       | n      => raise UnknownFlagError n
   
   fun field_mod m =
       case m of 0x0001 => "public"
               | 0x0002 => "private"
	       | 0x0004 => "protected"
	       | 0x0008 => "static"
	       | 0x0010 => "final"
	       | 0x0040 => "volatile"
	       | 0x0080 => "transient"
	       | n      => raise UnknownFlagError n
   
   fun cmod2flag m =
       case m of 0x0001 => ACCpublic
	       | 0x0010 => ACCfinal
	       | 0x0020 => ACCsuper
	       | 0x0200 => ACCinterface
	       | 0x0400 => ACCabstract
	       | n      => raise UnknownFlagError n

   fun mmod2flag m =
       case m of 0x0001 => ACCpublic
	       | 0x0002 => ACCprivate
	       | 0x0004 => ACCprotected
	       | 0x0008 => ACCstatic
	       | 0x0010 => ACCfinal
	       | 0x0020 => ACCsynchronized
	       | 0x0100 => ACCnative
	       | 0x0400 => ACCabstract
	       | 0x0800 => ACCstrictfp
	       | n      => raise UnknownFlagError n

   fun fmod2flag m = 
       case m of 0x0001 => ACCpublic
	       | 0x0002 => ACCprivate
	       | 0x0004 => ACCprotected
	       | 0x0008 => ACCstatic
	       | 0x0010 => ACCfinal
   	       | 0x0040 => ACCvolatile
	       | 0x0080 => ACCtransient
	       | n      => raise UnknownFlagError n
   

end;
