(*                                                                      *)
(* MLJVM - a Java Virtual Machine implemented in Standard ML            *)
(*                                                                      *)
(* http://homepages.inf.ed.ac.uk/stg/research/projects/MLJVM/           *)
(*                                                                      *)
(* Copyright (C) 2003 Stephen Gilmore and Laura Korte, The              *)
(* University of Edinburgh.                                             *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*                                                                      *)

structure MyClassFile :> MyClassFile =
struct

(****************************)
(**** START TOOLKIT CODE ****)
(****************************)


type local_var_info =
    {start  : Word16.word,
     length : Word16.word,
     name   : Constpool.index,
     desc   : Constpool.index,
     index  : Word16.word}

type line_number_info =
    {start : Word16.word,
     line  : Word16.word}

type exn_hdl =
    {start : Word16.word,
     stop  : Word16.word,
     entry : Word16.word,
     catch : Constpool.index option}

(* LK 20030531 - added inner_class_info *)
type inner_class_info =
    {inner_info  : Constpool.index option,
     outer_info  : Constpool.index option,
     inner_name  : Constpool.index option,
     inner_flags : Word16.word} 

datatype attribute =
    SRCFILE of {attr : Constpool.index,
		file : Constpool.index}
  | CONSTVAL of {attr  : Constpool.index,
		 value : Constpool.index}
  | CODE of {attr   : Constpool.index,
	     stack  : Word16.word,
	     locals : Word16.word,
	     code   : Word8Vector.vector,
	     hdls   : exn_hdl list,
	     attrs  : attribute list}
  | EXNS of {attr : Constpool.index,
	     exns : Constpool.index list}
  | LINENUM of {attr  : Constpool.index,
		lines : line_number_info list}
  | LOCALVAR of {attr : Constpool.index,
		 vars : local_var_info list}
  | INNER of {attr    : Constpool.index,	(* LK 20030531 - added INNER *)
              classes : inner_class_info list}
  | ATTR of {attr : Constpool.index,
	     info : Word8Vector.vector}

type member =
    {flags : Word16.word,
     name  : Constpool.index,
     desc  : Constpool.index,
     attrs : attribute list}

type class_file =
    {magic   : Word32.word,
     minor   : Word16.word,
     major   : Word16.word,
     pool    : Constpool.pool,
     flags   : Word16.word,
     this    : Constpool.index,
     super   : Constpool.index option,
     ifcs    : Constpool.index list,
     fields  : member list,
     methods : member list,
     attrs   : attribute list}


(****************************)
(***** END TOOLKIT CODE *****)
(****************************)

fun byte2string b = (Int.toString o Word8.toInt) b ^ "--"
fun my_map b2s v = let val size = Word8Vector.length v
                       val trans = b2s o Word8Vector.sub
                       fun my_map2 b2s v 0 = "\n" ^ trans (v,0)
		         | my_map2 b2s v n = (my_map2 b2s v (n-1)) ^ trans (v,n)
		   in  my_map2 b2s v (size-1) ^ "\n"
		   end
end
