(*                                                                      *)
(* MLJVM - a Java Virtual Machine implemented in Standard ML            *)
(*                                                                      *)
(* http://homepages.inf.ed.ac.uk/stg/research/projects/MLJVM/           *)
(*                                                                      *)
(* Copyright (C) 2003 Stephen Gilmore and Laura Korte, The              *)
(* University of Edinburgh.                                             *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*                                                                      *)

structure RuntimePool =
struct

exception InvalidEntry of string
fun entryError s = raise InvalidEntry("Constpool." ^ s)

datatype index = IDX of int

val maxPool = 0xffff

fun makeIndex k = if 0 <= k andalso k < maxPool then IDX k
		  else raise Overflow

fun indexValue (IDX i) = i

datatype runentry = RCPmethodref of int
                  | RCPfieldref of { fieldIndex : int
		                   , index : index }
                  | RCPclassref of { index : index
		                   , classIdx : int}

datatype entry = CP of Constpool.entry
               | RCP of runentry

type pool = entry Array.array

fun lookup pool (IDX i) =
    if 1 <= i
    then Array.sub(pool, i)
    else raise Subscript

val cp2rcp = (makeIndex o Constpool.indexValue)

exception FormatError of int;
fun extract_class (pool:pool) idx =
case (lookup pool idx) of
     CP (Constpool.CPclass idx2) => ( case (lookup pool (cp2rcp idx2)) of
                                        CP (Constpool.CPutf8 s) => s
			              | n                       => raise FormatError 1)
   | n                           => raise FormatError 2


fun extract_name (pool:pool) idx =
case (lookup pool idx) of CP (Constpool.CPutf8 s) => s
	                | n             => raise FormatError 8


end;

