(* Int32.sml
 *
 * Peter Bertelsen
 * November 1997
 *)

datatype int = INT of Word32.word

local
    val min = Word8Vector.fromList [0wx80, 0w0, 0w0, 0w0]
    val max = Word8Vector.fromList [0wx7f, 0wxff, 0wxff, 0wxff]
in
    val minInt = SOME (INT (valOf(Word32.fromBytes min)))
    val maxInt = SOME (INT (valOf(Word32.fromBytes max)))
end

fun fromInt k = INT (Word32.fromInt k)

fun toInt (INT n) = Word32.toIntX n

fun fromBytes bs =
    (case Word32.fromBytes bs of
	 NONE   => NONE
       | SOME n => SOME (INT n))

fun toBytes (INT n) = Word32.toBytes n

fun compare (INT x, INT y) = Word32.compareX(x, y)

fun emit out (INT n) = Word32.emit out n

fun scan src =
    (case Word32.scan src of
	 NONE   => NONE
       | SOME n => SOME (INT n))

fun add (INT w1) (INT w2) =
    let val b1  = Word32.toBytes w1
	val b2  = Word32.toBytes w2
	val l1  = Word8Vector.foldr op:: [] b1
	val l2  = Word8Vector.foldr op:: [] b2
	val max = Word.fromInt 256
	fun f(x,y,(z,c)) =
	    let val i = Word.+ ( Word.+ ( Word8.toLargeWord x,
	                                  Word8.toLargeWord y )
			       , Word.fromInt c )
	    in  if Word.< (i,max)
	        then (Word8.fromLargeWord i::z,0)
		else (Word8.fromLargeWord i::z,1)
	    end
        val result = (fromBytes o Word8Vector.fromList o #1)
	             (ListPair.foldr f ([],0) (l1,l2))
    in  valOf result
    end

fun negation (INT w2) =
    let	val b2     = Word32.toBytes w2
        val max1   = Word8.fromInt 255
        val max2   = Word8.fromInt 256
	val v1     = Word8Vector.extract(b2,0,SOME 3)
        val v1     = Word8Vector.map (fn x => Word8.- (max1,x)) v1
	val v2     = Word8Vector.extract(b2,3,NONE)
	val v2     = Word8Vector.map (fn x => Word8.- (max2,x)) v2
	val v      = Word8Vector.concat [v1,v2]
    in  (valOf o fromBytes) v
    end

fun subtract i1 i2 = 
    if i2 = (fromInt 0)
    then i1
    else add i1 (negation i2)

fun parseU1a v n =
    let val new_index = n+1
        fun i2i (tokens,n) = (Int.toString o Word8.toIntX o Word8Vector.sub) (tokens, n)
    in  (i2i (v,n) ^ " ", new_index)
    end

fun parseU1b v n =
    let val new_index = n+1
        fun i2i (tokens,n) = (Int.toString o Word8.toInt o Word8Vector.sub) (tokens, n)
    in  (i2i (v,n) ^ " ", new_index)
    end

fun <*> parser (result,vector_index,number) vector =
        if number > 0
        then let val (s,new_vi) = parser vector vector_index
	     in  <*> parser (result ^ s, new_vi, number-1) vector
	     end
	else (result,vector_index)

fun show (INT w) =
    let val b      = Word32.toBytes w
        val lb     = Word8Vector.length b
        val (s,i)  = <*> parseU1a ("",0,lb) b
    in  print (s ^ "\n")
    end

fun show2 (INT w) =
    let val b      = Word32.toBytes w
        val lb     = Word8Vector.length b
        val (s,i)  = <*> parseU1b ("",0,lb) b
    in  print (s ^ "\n")
    end

fun multiply i1 i2 =
    let val zero = fromInt 0
        val one  = fromInt 1
	fun aux i1 i2 imem =
            if   i1 = zero
            then imem
	    else aux (subtract i1 one) i2 (add imem i2)
    in  aux i1 i2 zero
    end
